/* Selects regions based on a minimum average flux level */


int applyfixedregions(int *imgnum, int *xdim, int *ydim, float ****flux, float **rms, int ***regionarray, float ***regflux, int *regnumber, float ***fluxerror, int averaged, float **regionlocx, float **regionlocy, int sigmalevel, int titles, int labels, int axismarks, int fixedregnum, int fixedregref, int *regionsset, float *fluxcalerror, float onsourcenoise) {


  int a, i, j, m, tmp_regnum, *reginten, *regionsize;
  int intenscale = 0;
  int intencount = 0;
  float **fracerror;

  char specmaptitle[32], xlabel[8], ylabel[8] ;

 // Declaring PGPLOT prototypes

  int cpgopen(const char *device);
  void cpgclos(void);
  void cpgenv(float xmin, float xmax, float ymin, float ymax, int just, int axis);
  void cpglab(const char *xlbl, const char *ylbl, const char *toplbl);
  void cpgpt1(float xpt, float ypt, int symbol);
  void cpgsci(int ci);


  // Set the number of regions to be the same
  regnumber[fixedregnum] = regnumber[fixedregref];

  // Allocating memory

  reginten = (int *)calloc(xdim[fixedregnum]*ydim[fixedregnum], sizeof(int));
  regionsize = (int *)calloc(xdim[fixedregnum]*ydim[fixedregnum], sizeof(int));
  fracerror = (float **)calloc(xdim[fixedregnum]*ydim[fixedregnum], sizeof(float *));

  regflux[fixedregnum] = (float **)calloc(imgnum[fixedregnum], sizeof(float *));
  regionarray[fixedregnum] = (int **)calloc(xdim[fixedregnum], sizeof(int *));
  fluxerror[fixedregnum] = (float **)calloc(imgnum[fixedregnum], sizeof(float *));

  // If this set already has regions, free them up
  if (regionsset[fixedregnum] == 0) {
    free(regionlocx[fixedregnum]);
    free(regionlocx[fixedregnum]);
  }

  regionlocx[fixedregnum] = (float *)calloc(xdim[fixedregref]*ydim[fixedregref], sizeof(float));
  regionlocy[fixedregnum] = (float *)calloc(ydim[fixedregref]*xdim[fixedregref], sizeof(float));


  for (a=0; a<imgnum[fixedregnum]; a++) {
    regflux[fixedregnum][a] = (float *)calloc(xdim[fixedregnum]*ydim[fixedregnum], sizeof(float));
    fluxerror[fixedregnum][a] = (float *)calloc(xdim[fixedregnum]*ydim[fixedregnum], sizeof(float));
    fracerror[a] = (float *)calloc(xdim[fixedregnum]*ydim[fixedregnum], sizeof(float));
  }


  // Copy the array from reference to target
  for (i=0; i<xdim[fixedregref]; i++){
    regionarray[fixedregnum][i] = (int *)calloc(ydim[fixedregnum], sizeof(int));

    for (j=0; j<ydim[fixedregref]; j++){

      regionarray[fixedregnum][i][j] = regionarray[fixedregref][i][j];

    }
  }

  // Copy over the location of the regions center pixel and the colours for mapping

  for (i=0; i<regnumber[fixedregref]; i++){

    regionlocx[fixedregnum][i] = regionlocx[fixedregref][i];
    regionlocy[fixedregnum][i] = regionlocy[fixedregref][i];

  }


  for (a=0; a<imgnum[fixedregnum]; a++){
    for (i=0; i<xdim[fixedregnum]; i++){
      for (j=0; j<ydim[fixedregnum]; j++){

	tmp_regnum = regionarray[fixedregnum][i][j];

	if (tmp_regnum >= 0) {

	regflux[fixedregnum][a][tmp_regnum] += flux[fixedregnum][a][i][j];
	regionsize[tmp_regnum]++;

	//printf("Reg Number: %d Reg Flux: %.2e \n", tmp_regnum, regflux[fixedregnum][a][tmp_regnum]);
	}
      }
    }
  }

  // Calculate the error of each region

  for (m=1; m<=regnumber[fixedregnum]; m++){

    for (a=0; a<imgnum[fixedregnum]; a++){

      fluxerror[fixedregnum][a][m] = sqrt( pow(regflux[fixedregnum][a][m] * fluxcalerror[a], 2) + (regionsize[m] * (pow(rms[fixedregnum][a] * onsourcenoise, 2) ) ) );

      fracerror[a][m] = fluxerror[fixedregnum][a][m] / regflux[fixedregnum][a][m];
    }
  }

  // Average the flux if set to do so

  if (averaged == 1) {

    for (i=0; i<regnumber[fixedregnum]; i++){

      for (j=0; j<imgnum[fixedregnum]; j++){

	regflux[fixedregnum][j][i] /= regionsize[i];
	fluxerror[fixedregnum][j][i] = regflux[fixedregnum][j][i] * fracerror[j][i];
      }
    }  
  }

  printf("Mapping the regions...");

  cpgopen("/xs");
  cpgsci(1);

  if (axismarks == 1) {
    cpgenv(0,xdim[fixedregnum],0,ydim[fixedregnum], 1, 0);
  }
  else {
    cpgenv(0,xdim[fixedregnum],0,ydim[fixedregnum], 1, -1);
  }

  if (titles == 1) { sprintf(specmaptitle,"Map of Adaptive Regions"); }
  else { sprintf(specmaptitle," ");  }

  if (labels == 1) {
  sprintf(xlabel,"Pixels");
  sprintf(ylabel,"Pixels");
  }
  else {
    sprintf(xlabel," ");
    sprintf(ylabel," ");
  }

  cpglab(xlabel, ylabel, specmaptitle);

  /*
  //For testing flux values
  for (i=0; i<imgnum; i++){
  for (j=1; j<regnumber[0]; j++){

  printf("Map: %d Region: %d Flux: %.3e \n", i, j, regflux[i][j]);
  
  }
  }
  */

  // Plot the regions to make a map

  for (i=4; i<(xdim[fixedregnum]-4); i++){
    for (j=4; j<(ydim[fixedregnum]-4); j++){
      
	if (regionarray[fixedregnum][i][j] == -1) {
	  intenscale = 0;
	}
	else if (regionarray[fixedregnum][i][j] == -2) {
	  intenscale = 0;
	}
	else if (regionarray[fixedregnum][i][j] == 0) {
	  intenscale = 0;
	}
	else {
	  intenscale = intencount;
	  intencount++;

	  if (intencount >=15) {
	    intencount = 0;
	  }

	}
      cpgsci(intenscale);
      cpgpt1(j,i,1);
    }
  }

  // Close off the plotting device so other functions can call it
  cpgclos();

  printf("\n\nTotal number of regions: %d\n\n", regnumber[fixedregnum]);


// Free up redundant memory usage

  for (a=0; a<imgnum[fixedregnum]; a++){
    free(fracerror[a]);
  }

  free(fracerror);
  free(reginten);
  free(regionsize);

  return 0;

}
