// Maps the chisquared of each regions best fit as a function of posision. Note the non-linear colour scale

int chisquaredmap(float *chisquared, int **regionarray, int xaxismax, int yaxismax, int imgnum, int border, float zoom, char maptitle[], int titles, int labels, int axismarks, int extwincontrol, int smoothmaps, int contours, int contourlevels, int contourcolour, int contourlinestyle, float **flux, int firstcontour, int contourlogs, char *output, double inject, double fieldstrength, int paramlabels, int model, float ra, float dec, float cellsize, int scaletype, float bmaj, float bmin, float bpa, int beamposx, int beamposy, int plotbeam, float wedgemultiplier, int usereduced, int xshift, int yshift) {

#include "smoothmaps.h"

  int mapscale(int scaletype, int istart, int iend, int jstart, int jend, int border, float cellsize, int axismarks, int labels, float ra, float dec, char *maptitle, int xshift, int yshift);

  int drawbeam(float xpos, float ypos, float bmaj, float bmin, float bpa);

  // Declare the local variables
  int i, j, istart, jstart, iend, jend;
  int intenscale = 0;
  float zoomedx, zoomedy, adj_maxrange;
  char str_inject[64];
  char str_field[64];
  char wedgelabel[64];

  int dof = 2;
  
  // Declare pgpplot prototypes
  int cpgopen(const char *device);
  void cpgclos(void);
  void cpgpt1(float xpt, float ypt, int symbol);
  void cpgsci(int ci);
  void cpgimag(const float *a, int idim, int jdim, int i1, int i2, int j1, int j2, float a1, float a2, const float *tr);
  void cpgctab(const float *l, const float *r, const float *g, const float *b, int nc, float contra, float bright);
  void cpgwedg(const char *side, float disp, float width, float fg, float bg, const char *label);
  void cpgscr(int ci, float cr, float cg, float cb);
  void cpgcont(const float *a, int idim, int jdim, int i1, int i2, int j1, int j2, const float *c, int nc, const float *tr);
  void cpgsls(int ls);
  void cpgmtxt(const char *side, float disp, float coord, float fjust, const char *text);

 if (extwincontrol != 1) { 
    cpgopen(output);
  }

  // Swap to background and forground colours to something sensible
  cpgscr(0, 1, 1, 1);
  cpgscr(1, 0, 0, 0);

  // Set plot colour
  cpgsci(1);


  // Calculate the zoomed start and end points
  zoomedx = xaxismax / zoom;
  zoomedy = yaxismax / zoom;

  istart = xshift + border + ( (xaxismax - (int)zoomedx) / 2);
  jstart = yshift + border + ( (yaxismax - (int)zoomedy) / 2);

  iend = xshift + xaxismax - ( ( (xaxismax - (int)zoomedx) / 2) + border);
  jend = yshift + yaxismax - ( ( (yaxismax - (int)zoomedy) / 2) + border);

  if (titles == 0) { sprintf(maptitle," ");  }


  mapscale(scaletype, istart, iend, jstart, jend, border, cellsize, axismarks, labels, ra, dec, maptitle, xshift, yshift);


  if (smoothmaps == 1) {

    // Do smooth mapping
    float *tmp_smootharray;
    float minrange = 1e23;
    float maxrange = -1e23;


    tmp_smootharray = (float *)calloc(xaxismax*yaxismax, sizeof(float));


    for (i=0; i<xaxismax; i++) {
      for (j=0; j<yaxismax; j++) {
	
	if (regionarray[i][j] < 0) { // Set void regions to a high value to create a white background
	  tmp_smootharray[i*xaxismax+j] = 1e24;
	}

	else {

	  if (usereduced == 1) {
	  tmp_smootharray[i*xaxismax+j] = chisquared[regionarray[i][j]]/(imgnum - dof);
	  }
	  else {
	    tmp_smootharray[i*xaxismax+j] = chisquared[regionarray[i][j]];
	  }

	  if (tmp_smootharray[i*xaxismax+j] < minrange) {
	    minrange = tmp_smootharray[i*xaxismax+j];
	  }

	  if (tmp_smootharray[i*xaxismax+j] > maxrange) {
	    maxrange = tmp_smootharray[i*xaxismax+j];
	  }
	}
      }
    }


    if (usereduced == 1) {
      sprintf(wedgelabel, "Reduced Chi-Squared");
    }
    else {
      sprintf(wedgelabel, "Chi-Squared");
    }

    adj_maxrange = maxrange * wedgemultiplier;

    cpgctab(rl,rr,rg,rb,11,contra,bright);
    cpgwedg("RI", 1.5, 5.0, minrange, adj_maxrange, wedgelabel);
    cpgimag(tmp_smootharray,xaxismax,yaxismax,istart,iend,jstart,jend,minrange,adj_maxrange,trans);

    free(tmp_smootharray);
  }

  else {

    // Do the old style mapping
    for (i=istart; i<iend; i++){
      for (j=jstart; j<jend; j++){
	if (regionarray[i][j] <= 0) {
	  intenscale = 0;
	}
	else {
	  //printf("Bestage: %d", regionarray[i][j]);

	  if ((chisquared[regionarray[i][j]] / (imgnum -2)) < 0.5) {
	    intenscale = 1;
	  }
	  else if ( ((chisquared[regionarray[i][j]] / (imgnum -2)) >= 0.5) && ((chisquared[regionarray[i][j]] / (imgnum -2)) < 2) ) {
	    intenscale = 2;
	  }
	  else if ( ((chisquared[regionarray[i][j]] / (imgnum -2)) >= 2) && ((chisquared[regionarray[i][j]] / (imgnum -2)) < 4) ) {
	    intenscale = 3;
	  }
	  else if ( ((chisquared[regionarray[i][j]] / (imgnum -2)) >= 4) && ((chisquared[regionarray[i][j]] / (imgnum -2)) < 6) ) {
	    intenscale = 4;
	  }
	  else if ( ((chisquared[regionarray[i][j]] / (imgnum -2)) >= 6) && ((chisquared[regionarray[i][j]] / (imgnum -2)) < 8) ) {
	    intenscale = 5;
	  }
	  else if ( ((chisquared[regionarray[i][j]] / (imgnum -2)) >= 8) && ((chisquared[regionarray[i][j]] / (imgnum -2)) < 10) ) {
	    intenscale = 6;
	  }
	  else if ( ((chisquared[regionarray[i][j]] / (imgnum -2)) >= 10) && ((chisquared[regionarray[i][j]] / (imgnum -2)) < 12) ) {
	    intenscale = 7;
	  }
	  else if ((chisquared[regionarray[i][j]] / (imgnum -2)) > 12) {
	    intenscale = 8;
	  }
	  else {
	    intenscale = 0;
	  }

	}

	cpgsci(intenscale);
	cpgpt1(j,i,1);

      }
    }

  }


  if (contours == 1) {

    float *tmp_contourarray;
    float minrange = 1e32;
    float maxrange = -1e32;
    float currentcont;


    cpgsci(contourcolour);
    cpgsls(contourlinestyle);

    tmp_contourarray = (float *)calloc(xaxismax*yaxismax, sizeof(float));

    for (i=0; i<xaxismax; i++) {
      for (j=0; j<yaxismax; j++) {

       	if (regionarray[i][j] > 0) { // Stop non-included regions from creating a false minimum

	  tmp_contourarray[i*xaxismax+j] = flux[i][j];

	  if (tmp_contourarray[i*xaxismax+j] < minrange) {
	    minrange = tmp_contourarray[i*xaxismax+j];
	  }

	  if (tmp_contourarray[i*xaxismax+j] > maxrange) {
	    maxrange = tmp_contourarray[i*xaxismax+j];
	  }
	}

	else {
	  tmp_contourarray[i*xaxismax+j] = 0.0;
	}

      }
    }

    for (i=firstcontour; i<=contourlevels; i++) {

      if (contourlogs == 1) {
	currentcont = pow(10, log10(minrange) + (i*(log10(maxrange)-log10(minrange))/contourlevels) );
      }
      else {
	currentcont = minrange + i*(maxrange-minrange)/contourlevels;
      }

      cpgcont(tmp_contourarray, xaxismax, yaxismax, istart, iend, jstart, jend, &currentcont, -1, trans);
    }

    free(tmp_contourarray);
  }


  if (paramlabels == 1) {

    if (model == 1) {
      cpgmtxt("B", -3.5, 0.65, 0.0, "JP Model");
    }
    if (model == 2) {
      cpgmtxt("B", -3.5, 0.65, 0.0, "KP Model");
    }
    if (model == 3) {
      cpgmtxt("B", -3.5, 0.65, 0.0, "Tribble Model (JP)");
    }
    else if (model == 4) {
      cpgmtxt("B", -3.5, 0.65, 0.0, "Tribble Model (KP)");
    }

    sprintf(str_inject, "Injection index: %.2f", inject);
    sprintf(str_field, "B-Field: %.2e T", fieldstrength);

    cpgmtxt("B", -2.5 , 0.65, 0.0, str_inject);
    cpgmtxt("B", -1.5 , 0.65, 0.0, str_field);
  }

  // Plot the beam if the option is set to on
  if (plotbeam == 1) {
    int beamposi = istart-border+(bmaj/cellsize)+beamposx;
    int beamposj = jstart-border+(bmaj/cellsize)+beamposy;

    // This must be after the main plotting to create an overlay
    drawbeam(beamposi, beamposj, bmaj/cellsize, bmin/cellsize, bpa);
  }

  if (extwincontrol != 1) {
    cpgclos();
  }

  return 0;

}
