/*
List of commands available for use and their descriptions
*/

#include <stdio.h>

int help() {
  printf("\nListed available commands.\nAll commands assume entry at the main BRATS prompt with the exception of the 'esc' command which can be used to escape the current task.\n\n");

  printf("quit - Exits the program\n");
  printf("load - Loads a new dataset\n");
  printf("esc - Escape the current task\n");
  printf("ls - List the contents of the current directory (Note: For options such as -r, use the 'shell' task)\n");
  printf("shell - Task to allow running of shell commands\n");
  printf("list - Lists all of the currently loaded datasets\n");
  printf("props - Lists the general properities of a dataset\n");
  printf("setregions - Create adaptive regions of a specific or all datasets\n");
  printf("sigma - Change the level of source detection\n");
  printf("zoom - Set the zoom level for flux, spectral index and curvature maps (DEFAULT = 1)\n");
  printf("border - Set the border size in pixels for flux, spectral index and curvature maps (DEFAULT = 4)\n");
  printf("titles - Turn on and off titles in maps and plots\n");
  printf("labels - Turn on and off axis lables in maps and plots\n");
  printf("axismarks - Turn on and off axis markings in maps and plots\n");
  printf("maptomap - Change the limit at which map to map variations will cause flagging as a decimal of the flux. If there is a large break in frequency between maps, you may need to turn this off or set a high value. (DEFAULT = -1 [OFF])\n");
  printf("hotpixels - Change the limit at which pixels will be considered hot and cold variations in flux will cause flagging as a decimal of the flux (DEFAULT = 0.2)\n");
  printf("averegions - Turn on and off the averaging of flux within regions (DEFAULT = OFF)\n");
  printf("signaltonoise - Change the signal to noise limit for region selection (DEFAULT = 250)\n");
  printf("searcharea - Change the maximum area a region can be before it is flagged (DEFAULT = 50)\n");
  printf("specindex - Calculate and map the spectral indices by region of a given data set\n");
  printf("printindex - Turn on and off the printing of spectral index values as they are calculated (DEFAULT = OFF)\n");
  printf("fixedregions - Applies the regions of one data set to another using pixel coordinates\n");
  printf("fluxmaps - Produce a flux map using either raw data or adaptive regions for a given dataset\n");
  printf("plotflux - Plot flux and a fuction of frequency");
  printf("autoscale - Turn on and off autoscaling of all plot axis, and set or change user defined values\n");
  printf("printinc - Turn on and off the printing (to screen) of regions with an increasing flux with increasing frequency\n");
  printf("uselogs - Turn on and off the use of logarithmic values for plots\n");
  printf("skip - Set a value of how many regions to skip in a plot (i.e. only plot every nth region)\n");
  printf("fitpoly - Fit a polynomial to a data set. Regions must be set\n");
  printf("plotres - The number of points used when plotting a curve (DEFAULT = 1000, MINIMUM 10)\n");
  printf("setpoly - Define the order of polynomial to use for fitting (DEFAULT = 2)\n");
  printf("curvecon - Switch the sign convention used when fitting curves (DEFAULT = OFF)\n");
  printf("fluxcalerror - Change the flux calibration error for a single or range of maps and datasets. A region selection command must be re-run after any changes to ensure the region errors are updated.\n");
  printf("bestguess - Change the starting point of the beat fitting spectal index search (DEFAULT = 1.0)\n");
  printf("specrange - Change the spectral index range over which the best fitting  index will be searched (DEFAULT = 0.5 [+-0.5])\n");
  printf("normrange - Change the normalisation range over which the best fitting spectral index will be searched (DEFAULT = 0.2 [2 per cent])\n");
  printf("symbol - Chance the symbol which is used to mark values when creating plots (DEFAULT = 1)\n");
  printf("specres - The number of spectral indices attempted between the maximum and minimum spectral index range when finding best fits (spread linearly, DEFAULT = 100) ");
  printf("fluxres - The number of flux normalisation values attempted between the maximum and minimum normalisation range when finding best fits (spread linearly, DEFAULT = 100)\n");
  printf("onsource - Change the value by which the RMS is multiplied for the on-source noise (DEFAULT = 1)\n");
  printf("minmodelfreq - Change the minimum frequency in Hz for the plotting of example models\n");
  printf("maxmodelfreq - Change the maximum frequency in Hz for the plotting of example models\n");
  printf("ageres - Change the resolutions of the ages attempted when fitting models e.g. 10 = 0.1 step size for each myear (DEFAULT = 1)\n");
  printf("myears - Change the number of model ages to attempt e.g. 10 gives ages between 0 and 9 MYears (DEFAULT = 10)\n");
  //printf("normres - Change the number of normalisation values tested for each spectral age (DEFAULT = 10)\n");
  printf("plotcurvespec - Plot curvature against spectral index for a given dataset\n");
  printf("plotmodelobs - Plot the flux values of both the model and observed data as a function of frequency\n");
  printf("modelres - Change the number of points for which the model data will be calculated (DEFAULT = 100)\n");
  printf("specagemap - Plot a map of the spectral age as a function of position (from fitjpmodel)\n");
  printf("chisquaredmap - Plot a map of the chi-squared values as a function of position (from fitjpmodel)\n");
  printf("casadata - Switch between CASA and AIPS formats for the 'load' command (DEFAULT = AIPS)\n");
  printf("levels - The number of (age) levels deep to go when model fitting (DEFAULT = 5)\n");
  printf("printresults - Turn on and off the printing of individual results when model fitting. The averages will still be printed on completion (DEFAULT = OFF)\n");
  printf("smoothmaps - Switch between smooth maps and the original style with harder cuts (DEFAULT = ON)\n");
  printf("contours - Turn on and off the printing of contours (DEFAULT = OFF)\n");
  printf("contourlevels - Set the number of contours to plot between min and max values (DEFAULT = 8)\n");
  printf("contourcolour - Set the colour of the contour lines (DEFAULT = BLACK)\n");
  printf("contourlinestyle - Set the style of contour lines (DEFAULT = SOLID)\n");
  printf("contourlogs - Set whether to use linear of logarithim spacings between contours (DEFAULT = LOG)\n");
  printf("firstcontour - The level of the first contour to plot (above the minimum) (DEFAULT = 1)\n");
  printf("export - Turn on and off the exporting of data (DEFAULT = OFF)\n");
  printf("imageloc - Set the location for data to be exported (DEFAULT = ./images)\n");
  printf("imagetype - Set the format of exported data (DEFAULT = PNG)\n");
  printf("fluxlogs - Use logs for flux values when mapping (DEFAULT = OFF)\n");
  printf("fluxcut - Turn on and off the user defined cut level set by 'usercut' (DEFAULT = OFF)\n");
  printf("usercut - Set the minimum flux cut off when mapping flux values. 'fluxcut' must also be turned on for this value to be applied (DEFAULT = 1e-6)\n");
  printf("wedgemultiplier - Multiplier to apply to the maximim value for a given dataset that defines the upper limit of the wedge colour. This is useful in avoiding real values from taking the background colour. Good values are around 1.15 for linear maps and around 2 for log maps (DEFAULT = 1.15)\n");

  printf("bfield - Set the magnetic field strength to use in models. (DEFAULT = 1e-9)\n");
  printf("injectionindex - Set the injection index to use in models. (DEFAULT = 0.6)\n");
  printf("modelpower - Set the injection index to use in models based on the model power, where alpha = (power-1)/2 (DEFAULT = 2.2)\n");
  printf("colourcolour - Create a colour-colour plot of a given dataset\n");
  printf("alwayszeroage - Turn on and off the setting of the minimum age range to zero for spectral ageing maps. Note: this only adjusts the wedge and colour range, not the data values. (DEFAULT = ON)\n");
  printf("plotkpmodel - Plot an example KP model between minmodelfreq and maxmodelfreq for arbitrary normalisation.\n");
  printf("paramlabels - Turn on and off parameter labels for model and statistical maps. (DEFAULT = ON)\n");
  printf("printreject - Turn on and off the output whether a model should be rejected based on if the average chi-squared is above the 90 per cent confidence interval. Note: If there are a number of regions where you expect a bad fit e.g. from dynamic range issues, the average may not be a suitable value to use.\n");
  printf("gmin - Set the minimum value of gamma for model fitting (DEFAULT = 10)\n");
  printf("gmax - Set the maximum value of gamma for model fitting (DEFAULT = 100000)\n");
  printf("posmap - Set which map should be used as reference for co-ordinates when plotting maps in WCS (DEFAULT = 0)\n");
  printf("scaletype - Set the scale to be used when mapping. Options are: Pixels, Arcseconds (from centre of map), Arcseconds in degrees, minutes, seconds format or WCS. (DEFAULT = WCS)\n");
  printf("beampos - Set the position to plot the beam when making maps (DEFAULT X = 10, Y = 10)\n");
  printf("beam - Turn on and off the plotting of the beam to maps (DEFAULT = ON)\n");
  printf("exportdata - Exports numerical data such as spectral ages to a text file at a location specified by dataloc\n");
  printf("dataloc - Specifies the location to export numerical data. (DEFAULT = ./data) \n");
  printf("usereduced - Turn on anf off the use of reduced chi-squared values for plotting and mapping (DEFAULT = ON)\n");
  printf("conflevels - Print to screen chi-squared confidence levels for a given degrees of freedom. If datasets are loaded, the DoF can be automatically calculated. If no datasets are loaded or 888 is entered at the dataset selection screen, the user will be prompted to enter the desired number of DoF\n");
  printf("modelmyears - Set the maximum age to plot when outputting models (e.g. plotjpmodel). Models will output from 0 Myears to this number at intervals of 1 Myear\n");
  printf("setsingleregion - Sets a single region (opposed to many adaptive regions) to included any flux above sigma*RMS within the area of the region file. Note: This should not be used when region size << beamsize. The error checking performed when setting regions in this way is also much less rigorous than with the setregions command e.g. maptomap and hotpixels are not applied. We leave it to the user to ensure the region is large enough, sits in a sensible region and reaches the desired signal to noise ratio.\n");
  printf("findinject - Run a model fit (or multiple model fits) with varying injection indices in an attempt to find the best fitting injection index. Warning: This function can take a VERY long time to run, especially if multiple models over multiple datasets are being attempted.\n");
  printf("mininject - Set the minimum injection index be tested with findinject (DEFAULT = 0.5)\n");
  printf("maxinject - Set the maximum injection index be tested with findinject (DEFAULT = 1.0)\n");
  printf("injectintervals - The number of intervals between mininject and maxinject to be tested by findinject. n + 1 intevals are tested to find the best fit e.g. with mininject 0.5 and maxinject 1.0, n = 10 will result in steps of 0.5 (DEFAULT = 10)\n");
  printf("viewerrors - Displays the currently set values for the flux calibration errors\n");
  printf("telescope - Change the default flux calibration errors set when loading a map to a given telescope, or, a fixed user value.\n");
  printf("plotjpmodel - Plot an example JP model between minmodelfreq and maxmodelfreq for an arbitrary normalisation.\n");
  printf("plotkpmodel - Plot an example KP model between minmodelfreq and maxmodelfreq for an arbitrary normalisation.\n");
  printf("plotjptribble - Plot an example Tribble model with a JP electron energy distribution model between minmodelfreq and maxmodelfreq for an arbitrary normalisation.\n");
printf("plotkptribble - Plot an example Tribble model with a KP electron energy distribution model between minmodelfreq and maxmodelfreq for an arbitrary normalisation.\n");
  printf("fitjpmodel - Fit a JP model to a dataset and calculate the corresponding chi-squared values.\n");
  printf("fitkpmodel - Fit a KP model to a dataset and calculate the corresponding chi-squared values.\n");
  printf("fitjptribble - Fit a Tribble model with a JP electron energy distribution and calculate the corresponding chi-squared values.\n");
  printf("fitkptribble - Fit a Tribble model with a KP electron energy distribution and calculate the corresponding chi-squared values.\n");
  printf("combineimages - Combines a set of radio maps to a common (user defined) frequency in the image plane.\n");
  printf("diffmap - Subtracts the flux of one fits file from another and outputs a third fits image. subtraction can either be full map, or, confined to the regions of a dataset.\n");
  printf("scaleflux - Scales the raw flux values by a given scaling factor. Either single scaling value or interpolation / extrapolation of two values can be used for either all or a subsection of maps. Region selection should be re-run after the scaling has been applied.\n");
  printf("shiftimages - Shifts the centre of output images by a specified amount. This command only changes the output images (either on screen or exported), not the raw data.\n");


  printf("\n");
  return 0;

}
