/*

  Loads a series of maps. Must be run first before most other tasks.

  Requires the file "chomp.h", "fluxmap.h" and <funtools.h> plus the standard headers to have been previously declared

*/

//Definitions
#ifndef MAXCMDLENGTH
#define MAXCMDLENGTH 1024
#endif

#ifndef MAXNUMFILES
#define MAXNUMFILES 50
#endif

#ifndef GFACTOR
#define GFACTOR (2.0*sqrt(2.0*log(2.0)))
#endif


// Declare the function prototypes

int syscom(const char[]);

int fluxmap(float ****flux, float **rms, int *xdim, int *ydim, int mapnumber, float **maxflux, float **minflux, int currentset, int border, float zoom, int sigmalevel, char fluxmaptitle[128], int titles, int labels, int axismarks, int extwincontrol, int smoothmaps, int fluxlogs, int fluxcut, float usercut, float wedgemultiplier, char *output, float ra, float dec, float cellsize, int scaletype, float bmaj, float bmin, float bpa, int beamposx, int beamposy, int plotbeam, int xshift, int yshift);

int load(char dirname[], int currentset, int numdatasets, int *imgnum, int *xdim, int *ydim, float *bmaj, float *bmin, float *bpa, float *beamarea, float **frequency, float **frequencyobs, char regname[], char bgname[], float ****flux, float **rms, int sigmalevel, int **fluxorder,float zoom, int border, int titles, int labels, int axismarks, char tmp_maptarget[], float **yaxismax, float **yaxismin, int casadata, int fluxlogs, int fluxcut, float usercut, float wedgemultiplier, float **ra, float **dec, float *cellsize, float scaletype, int posmap, int beamposx, int beamposy, int plotbeam, float ***bgbuff, int xshift, int yshift, float *redshift) {


  //Declare the variables for this task

  register char **fitsfiles, **bgfiles;

  float celltol = 0.000000000000001; // This needs to be made variable

  int usereg, required, i, j, a, got, m, *tmp_xdim, *tmp_ydim, count, tmp_casadata;
  float *tmp_bmaj, *tmp_bmin, *tmp_bpa, *tmp_beamarea, bgflux, **imgbuff, curminfreq, curmaxfreq, tmp_redshift;
  double delt, delt2, rapix, decpix;
  char *fileloc, **units, *bgloc, **maptarget, fluxmaptitle[128], output[16];

  FITSHead head;
  FITSCard card;
  Fun *fitsconn, *bgconn;
  DIR *direct;

  char *str_casadata, str_compare_aips[8], str_compare_casa[8];
  strcpy(str_compare_aips, "AIPS");
  strcpy(str_compare_casa, "CASA");

  struct dirent *entry;

  //Get the directory name and format it
 
  required = 1;

  while (required == 1) {

    strcpy(dirname, "");

    fputs("Enter directory name: ", stdout);
    fflush(stdout);
    fgets(dirname, MAXCMDLENGTH, stdin);

    chomp(dirname);

    // Check a directory name has been enetered isn't over the maximum character limit
    if (strlen(dirname) >= MAXCMDLENGTH-1) {
      fprintf(stderr,"\n*** Error: Folder path must be less than %d characters long! ***\n\n", MAXCMDLENGTH);
      return(2);
    }
    else if (strlen(dirname) == 0) {
      printf("Please enter a directory name\n");
    }
    else if ((strstr(dirname, "esc") != NULL) && (strlen(dirname) == 3)) {
      printf("Escaping command...\n");
      return 100;
    }
    else if ((strstr(dirname, "ls") != NULL) && (strlen(dirname) == 2)) {
      syscom("ls");
    }
    else {
      required = 0;
    }
  }

  required = 1;

  while (required == 1) {

    //Get the background file name and format it
    fputs("Background file path: ", stdout);
    fflush(stdout);
    fgets(bgname, MAXCMDLENGTH, stdin);

    chomp(bgname);

    if (strlen(bgname) >= MAXCMDLENGTH-1) {
      fprintf(stderr,"\n*** Error: Folder path must be less than %d characters long! ***\n\n", MAXCMDLENGTH);
      return 2;
    }
    else if (strlen(bgname) == 0) {
      printf("Please enter a directory name\n");
    }
    else if ((strstr(bgname, "esc") != NULL) && (strlen(bgname) == 3)) {
      printf("Escaping command...\n");
      return 100;
    }
    else if ((strstr(bgname, "ls") != NULL) && (strlen(bgname) == 2)) {
      syscom("ls");
    }
    else {
      required = 0;
    }
  }

  required = 1;

  while (required == 1) {

    //Get the region file name and format it
    fputs("Region file path (Default: full map): ", stdout);
    fflush(stdout);
    fgets(regname, MAXCMDLENGTH, stdin);

    chomp(regname);
    if (strlen(regname) >= MAXCMDLENGTH-1) {
      fprintf(stderr,"\n*** Error: File path must be less than %d characters long! ***\n\n", MAXCMDLENGTH);
      return 2;
    }
    else if ((strstr(regname, "esc") != NULL) && (strlen(regname) == 3)) {
      printf("Escaping command...\n");
      return 100;
    }
    else if ((strstr(regname, "ls") != NULL) && (strlen(regname) == 2)) {
      syscom("ls");
    }
    else {
      //Set if a region file is being used or not
      if (strlen(regname) == 0) {
	usereg = 0;
	required = 0;
      }
      else {
	usereg = 1;
	required = 0;
      }
    }
  }

  required = 1;

  while (required == 1) {

    printf("Redshift of the target source (-1 to exit): ");

    if (scanf("%f", &tmp_redshift) == 0) {
		 
      printf("Invalid input, escaping command to protect the existing data...\n");
      return 100;
    }

    if (tmp_redshift < 0) {
      printf("Escaping command...\n");
      return 100;
    }

    else {
      redshift[currentset]=tmp_redshift;
      required = 0;
    }
  }


  // Put some space between the input and the output
  printf("\n");
  /* Check all the files and directories exists then put them in the correct format  */

  // Setting image counter and allocating memory
  imgnum[currentset] = 0;

  tmp_xdim = (int *)calloc(MAXNUMFILES, sizeof(int));
  tmp_ydim = (int *)calloc(MAXNUMFILES, sizeof(int));
  fitsfiles = (char **)calloc(MAXNUMFILES, sizeof(char *));
  bgfiles = (char **)calloc(MAXNUMFILES, sizeof(char *));
 
  
  //Open the directory
  direct = opendir(dirname);

  //Set the map counter to 0
  i = 0;

  //Check the directory exists
  if (direct != NULL) {

    //While there are maps to read...
    while ((entry = readdir(direct))) {
      if (strlen(entry->d_name) > MAXCMDLENGTH+1) {
	fprintf(stderr,"\n*** Error: File path must be less than %d characters long! ***\n\n", MAXCMDLENGTH);
	return 2;
      }
      //Do nothing if we are reading . or ..
      else if ((strcmp(entry->d_name, ".") == 0) || (strcmp(entry->d_name, "..") == 0)) {
	;
      }

      //Append the region and background file name to the map name...
      else {
	fitsfiles[i] = (char *)calloc((MAXCMDLENGTH+1),sizeof(char));
	strcpy(fitsfiles[i], entry->d_name);
	if (usereg != 0){
	  if( access(regname, F_OK) != -1 ) {
	    if( access(bgname, F_OK) != -1 ) {
	      bgfiles[i] = (char *)calloc((MAXCMDLENGTH+1),sizeof(char));
	      sprintf(bgfiles[i],"%s[@%s]", fitsfiles[i], bgname);     
	    }
	    else {
	      fprintf(stderr,"\n*** Error: Cannot open the background region file %s. ***\n\n", bgname);
	      return 404; 
	    }
	    // This must be second so as to pass the file name without the region appended!
	    sprintf(fitsfiles[i],"%s[@%s]", fitsfiles[i], regname);
	  }
	  else {
	    fprintf(stderr,"\n*** Error: Cannot open the region file %s. ***\n\n", regname);
	    return 404; 
	  }
	}

	// ... or if we are doing full map processing, just setup the appended background file name

	else {
	  if( access(bgname, F_OK) != -1 ) {
	    bgfiles[i] = (char *)calloc((MAXCMDLENGTH+1),sizeof(char));
	    sprintf(bgfiles[i],"%s[@%s]", fitsfiles[i], bgname);  
	  }   
	  else {
	    fprintf(stderr,"\n*** Error: Cannot open the background region file %s. ***\n\n", bgname);
	    return 404; 
	  }
	}

	//Increase the map counter
	i++;
	imgnum[currentset]++;

	// Check we havent exceeeded the maxmim number of files we are allowed 
	if (imgnum[currentset] >= MAXNUMFILES) {
	  fprintf(stderr,"\n*** Error: A maximum of %d files can currently be processed ***\n\n", MAXNUMFILES);
	  return 3;
	}
      }
    }

    closedir (direct);

  }

  //If we can't find or access the directory, kick back to the command prompt
  else {
    fprintf(stderr,"*** Error: Unable to locate FITS directory ***\n\n");
    return 404;
  }

  /* Start looping through the maps and getting the data */

  fileloc = (char *)calloc(MAXCMDLENGTH,sizeof(char));
  units = (char **)calloc(imgnum[currentset],sizeof(char *));
  maptarget = (char **)calloc(imgnum[currentset],sizeof(char *));
  tmp_bmaj = (float *)calloc(imgnum[currentset],sizeof(float));
  tmp_bmin = (float *)calloc(imgnum[currentset],sizeof(float));
  tmp_bpa = (float *)calloc(imgnum[currentset],sizeof(float));
  tmp_beamarea = (float *)calloc(imgnum[currentset],sizeof(float));
  bgloc = (char *)calloc(MAXCMDLENGTH,sizeof(char));
  ra[currentset] = (float *)calloc(imgnum[currentset],sizeof(float));
  dec[currentset] = (float *)calloc(imgnum[currentset],sizeof(float));

  frequency[currentset] = (float *)calloc(imgnum[currentset],sizeof(float));
  frequencyobs[currentset] = (float *)calloc(imgnum[currentset],sizeof(float));
  imgbuff = (float **)calloc(imgnum[currentset],sizeof(float *));
  bgbuff[currentset] =(float **)calloc(imgnum[currentset],sizeof(float *));

  for(i=0; i < imgnum[currentset]; i++) {
  
    //Make the full path name for each map
    sprintf(fileloc,"./%s/%s", dirname, fitsfiles[i]);
    sprintf(bgloc,"./%s/%s", dirname, bgfiles[i]);
    
    //Open the connection
    fitsconn = FunOpen(fileloc, "r", NULL);
 
    //Check we can access the files
    if (fitsconn == NULL){
      fprintf(stderr,"\n*** Error: Unable to open %s, please check the file exists and access permissions are correct. ***\n\n", fitsfiles[i]);
      return 404;
    }

    if( !(imgbuff[i] = FunImageGet(fitsconn, NULL, "bitpix=-32")) ) {
      fprintf(stderr, "\n*** Error: Unable to use ImageGet on %s, please check the file is in the correct format. ***\n\n", fitsfiles[i]);
      return 404;
    }

    /* Get map details and check that they are suitable for use */
  
    FunInfoGet(fitsconn, FUN_SECT_DIM1, &tmp_xdim[i], FUN_SECT_DIM2, &tmp_ydim[i], 0);
    if ((i > 0) && ((tmp_xdim[0] != tmp_xdim[i]) || (tmp_ydim[0] != tmp_ydim[i]))) {
      fprintf(stderr,"\n*** Error: Dimensions of %s do not match those of %s, the map sizes must be the same! ***\n\n", fitsfiles[i], fitsfiles[0] );
      return 4;
    }

    maptarget[i] = FunParamGets(fitsconn, "OBJECT", 0, NULL, &got);

    if (strcmp(maptarget[0], maptarget[i])) {
      fprintf(stderr,"\n*** Warning: The target of %s (as specified in the header) does not match that of %s! This may simply be due to different naming conventions between maps, but should be checked. ***\n\n", fitsfiles[i], fitsfiles[0]);
    }

    units[i] = FunParamGets(fitsconn, "BUNIT", 0, NULL, &got);
    if (strcmp("JY/BEAM", units[i])) {
      fprintf(stderr,"\n*** Error: The units used in the map %s (as specified in the header) are not JY/BEAM! ***\n\n", fitsfiles[i] );
      return 4;
    }

    frequencyobs[currentset][i] = FunParamGetd(fitsconn, "REFFREQ", 0, 0.0, &got);
    frequency[currentset][i] = frequencyobs[currentset][i] * (redshift[currentset] + 1);
    if (!got) {
      frequencyobs[currentset][i] = FunParamGetd(fitsconn, "CRVAL3", 0, 0.0, &got);
      frequency[currentset][i] = frequencyobs[currentset][i] * (redshift[currentset] + 1);
      if (!got) {
	fprintf(stderr,"\n*** Error: Unable to obtain the frequency of %s! ***\n\n", fitsfiles[i] );
	return 4;  
      }
    }

    /*
    frequency[currentset][i] = FunParamGetd(fitsconn, "CRVAL3", 0, 0.0, &got2);
    if (!got2) {
      fprintf(stderr,"\n*** Error: Unable to obtain the frequency of %s! ***\n\n", fitsfiles[i] );
      return 4;  
    }
    */

    if (i > 0) {
      for (a=0; a < i; a++) {
	if (frequencyobs[currentset][a] == frequencyobs[currentset][i]) {
	  fprintf(stderr,"\n*** Error: The frequency of the maps %s and %s is the same. This would lead to some odd results! ***\n\n", fitsfiles[a], fitsfiles[i]);
	  return 4; 
	}
      }
    }

    delt=FunParamGetd(fitsconn, "CDELT2", 0, 0.0, &got);
      
    if (got) {
      cellsize[currentset] = 3600 * delt;
    } else {
      fprintf(stderr,"\n*** Error: Unable to determine the pixel size! ***\n\n");
      return 4;
    }

    delt2=FunParamGetd(fitsconn, "CDELT1", 0, 0.0, &got);
  
    if (got) {

      // if (delt2!=-delt) {

      if ( ((abs(delt2) - abs(delt)) > celltol) || ((abs(delt) - abs(delt2)) > celltol) ) {
	fprintf(stderr,"\n*** Error: Pixels aren't square! (Failed on map frequency %.3e) ***\n\n", frequencyobs[currentset][i]);
	return 4;
      }
    } else {
      fprintf(stderr,"\n*** Error: Unable to determine the second pixel size! ***\n\n");
      return 4;
    }
 
    delt*=3600.0;

    FunInfoGet(fitsconn, FUN_HEADER, &head, 0);
      
    // At least all params below need adjusting to automatically accept to CASA data

    ra[currentset][i] = FunParamGetd(fitsconn, "CRVAL1", 0, 0.0, &got);
    if (!got) {
      fprintf(stderr,"\n*** Error: Unable to obtain the RA of %s! ***\n\n", fitsfiles[i] );
      return 4;  
    }

    dec[currentset][i] = FunParamGetd(fitsconn, "CRVAL2", 0, 0.0, &got);
    if (!got) {
      fprintf(stderr,"\n*** Error: Unable to obtain the Dec of %s! ***\n\n", fitsfiles[i] );
      return 4;  
    }

    // Adjust the RA and DEC so they are on the centre pixel

    rapix = FunParamGetd(fitsconn, "CRPIX1", 0, 0.0, &got);
    if (!got) {
      fprintf(stderr,"\n*** Error: Unable to obtain the pixel number for the RA of %s! ***\n\n", fitsfiles[i] );
      return 4;  
    }

    ra[currentset][i] += ( ( (tmp_xdim[i]/2) - rapix) * (cellsize[currentset] / 3600) );


    decpix = FunParamGetd(fitsconn, "CRPIX2", 0, 0.0, &got);
    if (!got) {
      fprintf(stderr,"\n*** Error: Unable to obtain the pixel number for the DEC of %s! ***\n\n", fitsfiles[i] );
      return 4;
    }

    dec[currentset][i] += ( ( (tmp_xdim[i]/2) - decpix) * ( cellsize[currentset] / 3600) );

    // Automatically determine the header type

    str_casadata=FunParamGets(fitsconn, "ORIGIN", 0, NULL, &got);

    //printf("str_casadata: %s\n", str_casadata);

    if (got) {
      if ( strncmp(str_casadata, str_compare_aips, 4) == 0 ) {
	tmp_casadata = 0;
      }
      else if ( strncmp(str_casadata, str_compare_casa, 4) == 0) {
	tmp_casadata = 1;
      }
      else {
	printf("Unable to automatically determine map origin. Assuming the manually set header type\n");
	tmp_casadata = casadata;
      }
    }
    else {
      printf("Unable to automatically determine map origin. Assuming the manually set header type\n");
      tmp_casadata = casadata;
    }

    if(tmp_casadata == 0) {

      // This deals with AIPS's crazy header encoding for beam parameters
      for(m=1; m<=head->ncard; m++){
	card = ft_cardnth(head,m);
	if( card && card->c && *card->c){
	  if (!strncmp(card->c, "HISTORY AIPS   CLEAN BMAJ",25)) {
	    sscanf(card->c+26,"%f",&tmp_bmaj[i]);
	    sscanf(card->c+44,"%f",&tmp_bmin[i]);
	    sscanf(card->c+61,"%f",&tmp_bpa[i]);
	    tmp_bmaj[i]*=3600; // Convert to arcseconds
	    tmp_bmin[i]*=3600;
	    tmp_bpa[i] += 90.0; // Convert from AIPS orientation to (0 is Y axis) to plot format (0 is X axis)
	    //printf("BMAJ: %.3f BMIN %.3f BPA %.2f \n", tmp_bmaj[i], tmp_bmin[i], tmp_bpa[i]);
	  }
	}
      }
    }
    else {
      tmp_bmaj[i] = FunParamGetd(fitsconn, "BMAJ", 0, 0.0, &got);
      tmp_bmin[i] = FunParamGetd(fitsconn, "BMIN", 0, 0.0, &got);
      tmp_bpa[i] = FunParamGetd(fitsconn, "BPA", 0, 0.0, &got);
      tmp_bmaj[i]*=3600;
      tmp_bmin[i]*=3600;
      tmp_bpa[i] += 90.0; // Check this for CASA
      //printf("BMAJ: %.3f BMIN %.3f \n", tmp_bmaj[i], tmp_bmin[i]);
    }

    if ( (fabs(tmp_bmaj[i]) - fabs(tmp_bmaj[0]) ) > (0.01*fabs(tmp_bmaj[0]) ) || (fabs(tmp_bmin[i]) - fabs(tmp_bmin[0]) ) > (0.01*fabs(tmp_bmin[0]) ) ) {
      fprintf(stderr,"\n*** Error: Beam parameters for %s and %s don't match! ***\n\n", fitsfiles[0], fitsfiles[i]);
      return 4;
    }

    if ( (fabs(tmp_bpa[i]) - fabs(tmp_bpa[0]) ) > (0.05 * fabs(tmp_bpa[0]) ) ) {
      fprintf(stderr,"\n*** Error: Beam position angle for %s and %s don't match! ***\n\n", fitsfiles[0], fitsfiles[i]);
      return 4;
    }

    // Parse the headers for encoding of the CLEAN beam size
    if (tmp_bmaj[i]>0.0) {
      tmp_beamarea[i]=2.0*PI*(tmp_bmaj[i]*tmp_bmin[i])/(GFACTOR*GFACTOR*delt*delt);
    } else {
      fprintf(stderr,"\n*** Failed to find CLEAN beam parameters -- Please check you have the correct header type selected (e.g. casadata) and the folder only contains radio maps? ***\n\n");
      return 4;
    }

    FunClose(fitsconn);

    // Get the info needed later for the background of this map

    bgconn = FunOpen(bgloc, "r", NULL);
   
    if (bgconn == NULL){
      fprintf(stderr,"\nError: Unable to open %s, please check the file exists and access permissions are correct.\nExiting...\n\n", fitsfiles[i]);
      exit(2);
    }

    if( !(bgbuff[currentset][i] = FunImageGet(bgconn, NULL, "bitpix=-32")) ) {
      fprintf(stderr, "Error: Unable to use ImageGet on %s, please check the file is in the correct format.\nExiting...\n\n", fitsfiles[i]);
    }

    FunClose(bgconn);
  }

  // Set the global dimensions to those found locally 
  xdim[currentset] = tmp_xdim[0];
  ydim[currentset] = tmp_ydim[0];
  bmaj[currentset] = tmp_bmaj[0];
  bmin[currentset] = tmp_bmin[0];
  beamarea[currentset] = tmp_beamarea[0];


  // Output what we found (More to do)

  printf("========================================================================\n");

  if (numdatasets == 0) {
    printf("Set %d was added or changed. There is now %d dataset in total\n", currentset, (numdatasets+1));
  }
  else {
    printf("Set %d was added or changed. There are now %d datasets in total\n", currentset, (numdatasets+1));
  }

  printf("There were %d maps found in the directory %s\n", imgnum[currentset], dirname);
  printf("Pixel size is %.2e arcsec\n", delt);
  printf("Beam area is %.2f pixels\n", beamarea[currentset]);
  printf("Beam major axis %.2f arcsec, minor axis %.2f arcsec\n", bmaj[currentset],bmin[currentset]);
  printf("The map dimensions are %d by %d\n", xdim[currentset], ydim[currentset]);

  printf("========================================================================\n\n");


  printf("Loading map data...\n");

  // Allocate memory to the overall flux, rms arrays and the min max flux values

  flux[currentset] = (float ***)calloc(imgnum[currentset],sizeof(float **)); // Maps
  yaxismax[currentset] = (float *)calloc(imgnum[currentset],sizeof(float));
  yaxismin[currentset] = (float *)calloc(imgnum[currentset],sizeof(float));

  for (a=0; a<imgnum[currentset]; a++) {

    flux[currentset][a] = (float **)calloc(xdim[currentset],sizeof(float *));

    for (i=0; i<xdim[currentset]; i++) {

      flux[currentset][a][i] = (float *)calloc(ydim[currentset],sizeof(float));
    }
  }

  rms[currentset] = (float *)calloc(imgnum[currentset],sizeof(float));

  //xaxismax = -1e+20;
  //xaxismin = 1e+20;

  float averagebackground;
  float rmsnumerator;

  // Find a RMS value for each map

  printf("\nRMS Noise (from background region):\n");

  for (a=0; a < imgnum[currentset]; a++) {
    bgflux = 0;
    count = 0;
    averagebackground = 0;
    rmsnumerator = 0;

    // Calculate the average flux
    for (i=0; i<xdim[currentset]; i++){
      for (j=0; j<ydim[currentset]; j++){
	bgflux += (bgbuff[currentset][a][i*xdim[currentset]+j] / beamarea[currentset]); // In per pixel format

	// If the flux is in the region (allowing for negative values) then increase the count
	if ( ( bgbuff[currentset][a][i*xdim[currentset]+j] < -1e-20) || ( bgbuff[currentset][a][i*xdim[currentset]+j] > 1e-20) ) {
	  count++;
	}
      }
    }

    //printf("Count: %d \n", count);

    averagebackground = bgflux / count;

    // Find the rms
    for (i=0; i<xdim[currentset]; i++){
      for (j=0; j<ydim[currentset]; j++){
	// Ensure we are in the region
	if ( ( bgbuff[currentset][a][i*xdim[currentset]+j] < -1e-20) || ( bgbuff[currentset][a][i*xdim[currentset]+j] > 1e-20) ) {
	  rmsnumerator += pow((bgbuff[currentset][a][i*xdim[currentset]+j] / beamarea[currentset]) - averagebackground, 2);

	}
      }
    }

    rms[currentset][a] =  sqrt(rmsnumerator / count); // This gives RMS in a per pixel format
    printf("%.2e Hz (Observed), %.2e Hz (Rest) = %.4e Jy/Pixel  (%.4e Jy/Beam)\n", frequencyobs[currentset][a], frequency[currentset][a], rms[currentset][a], rms[currentset][a]*beamarea[currentset]);
  }

  // Create some space so it is clearer
  printf("\n");

  // Loop through by map and find the source flux

  for (a=0; a < imgnum[currentset]; a++) {

    yaxismax[currentset][a] = -1e+20;
    yaxismin[currentset][a] = 1e+20;
 
    // Loop through by pixel
    for (i=(0); i<xdim[currentset]; i++){
 
      for (j=(0); j<ydim[currentset]; j++){
	    
	if (imgbuff[a][i*xdim[currentset]+j] > (sigmalevel * rms[currentset][a]) ) {

	  flux[currentset][a][i][j] = imgbuff[a][i*xdim[0]+j] / beamarea[currentset];


	  if (flux[currentset][a][i][j] > yaxismax[currentset][a]) {
	    yaxismax[currentset][a] = flux[currentset][a][i][j];
	  }

	  if (flux[currentset][a][i][j] < yaxismin[currentset][a]) {
	    yaxismin[currentset][a] = flux[currentset][a][i][j];
	  }
	}

	else {
	  flux[currentset][a][i][j] = 0.0;
	}
      }
    }

    // Clean up the image buffer as we go
    free(imgbuff[a]);
  }

  // Sort the order of the maps in terms of frequency

  fluxorder[currentset] =  (int *)calloc((imgnum[currentset]+1),sizeof(int));

  for (i=0; i < imgnum[currentset]; i++) {
    if (i == 0) {
      curminfreq = 1e-32;
    }
    else {
      curminfreq = frequency[currentset][fluxorder[currentset][i-1]];
    }
    curmaxfreq = 1e+32;

    for (a=0; a < imgnum[currentset]; a++) {
      if (a==0) {
	
	fluxorder[currentset][i] = a;
      }
      if ((frequency[currentset][a] > curminfreq) && (frequency[currentset][a] < curmaxfreq)) {
	curmaxfreq = frequency[currentset][a];
	fluxorder[currentset][i] = a;
      }
    }
  }

  sprintf(output,"/xs");


  sprintf(fluxmaptitle,"Flux of %s at %.2e Hz (%d Sigma Detection, Raw Data)", maptarget[fluxorder[currentset][0]], frequencyobs[currentset][fluxorder[currentset][0]], sigmalevel);

  fluxmap(flux, rms, xdim, ydim, fluxorder[currentset][0], yaxismax, yaxismin, currentset, border, zoom, sigmalevel, fluxmaptitle, titles, labels, axismarks, 0, 1, fluxlogs, fluxcut, usercut, wedgemultiplier, output, ra[currentset][posmap], dec[currentset][posmap], cellsize[currentset], scaletype, bmaj[currentset], bmin[currentset], bpa[currentset], beamposx, beamposy, plotbeam, xshift, yshift);

  //Free up the memory
  for (i=0; i<MAXNUMFILES; i++) {
    free(fitsfiles[i]);
    free(bgfiles[i]);
  }

  //Copy the target over to the transfering variable
  strcpy(tmp_maptarget, maptarget[0]);

  free(maptarget);
  free(fileloc);
  free(units);
  free(tmp_bmaj);
  free(tmp_bmin);
  free(tmp_beamarea);
  free(bgloc);
  free(imgbuff);
  free(tmp_xdim);
  free(tmp_ydim);
  free(fitsfiles);
  free(bgfiles);

  return 0;

}
