/* 
  Set the map scale and its labels.
*/

#include <stdio.h>
#include <stdlib.h>

int mapscale(int scaletype, int istart, int iend, int jstart, int jend, int border, float cellsize, int axismarks, int labels, float ra, float dec, char *maptitle, int xshift, int yshift) {

  void cpgenv(float xmin, float xmax, float ymin, float ymax, int just, int axis);
  void cpgaxis(const char *opt, float x1, float y1, float x2, float y2, float v1, float v2, float step, int nsub, float dmajl, float dmajr, float fmin, float disp, float orient);
  void cpgswin(float x1, float x2, float y1, float y2);
  void cpgtbox(const char *xopt, float xtick, int nxsub, const char *yopt, float ytick, int nysub);
  void cpglab(const char *xlbl, const char *ylbl, const char *toplbl);

  char xlabel[64];
  char ylabel[64];

  float conv_ra, conv_dec, xstartval, xendval, ystartval, yendval;

  float degtorad = 3.14159/180;
  float wholecirclesecs= 86400.0;


  if (scaletype == 1) {

    cpgenv(istart-border, iend+border, jstart-border, jend+border, 1, -1);

    // Determing the axis values for the given cellsize

    xstartval = 0 - ( ( ( (iend+border) - (istart-border) ) / 2 ) * cellsize);
    xendval = 0 + ( ( ( (iend+border) - (istart-border) ) / 2 ) * cellsize);

    ystartval = 0 - ( ( ( (jend+border) - (jstart-border) ) / 2 ) * cellsize);
    yendval = 0 + ( ( ( (jend+border) - (jstart-border) ) / 2 ) * cellsize);

    // Draw the X axis
    cpgaxis("N", istart-border, jstart-border, iend+border, jstart-border, xstartval, xendval, 0.0, 10, 0.7, 0.0, 0.4, 0.2, 0.0);

    // Now the Y axis
    cpgaxis("N", istart-border, jstart-border, istart-border, jend+border, ystartval, yendval, 0.0, 10, 0.0, 0.7, 0.4, -1.2, 0.0);

  }


  else if (scaletype == 2) {

    cpgenv(istart-border, iend+border, jstart-border, jend+border, 1, -1);

    // Determing the axis values for the given cellsize

    xstartval = 0 - ( ( ( (iend+border) - (istart-border) ) / 2 ) * cellsize);
    xendval = 0 + ( ( ( (iend+border) - (istart-border) ) / 2 ) * cellsize);

    ystartval = 0 - ( ( ( (jend+border) - (jstart-border) ) / 2 ) * cellsize);
    yendval = 0 + ( ( ( (jend+border) - (jstart-border) ) / 2 ) * cellsize);

    // Set to range in arcseconds
    cpgswin(xstartval, xendval, ystartval, yendval);

    // Draw the axis box
    if ( (axismarks == 1)){
      cpgtbox("BCZDNTSXYO", 0.0, 0.0, "BCZDNTSXYO", 0.0, 0.0);
    }

    else {
      cpgtbox("ZDNTSXYO", 0.0, 0.0, "ZDNTSXYO", 0.0, 0.0);
    }
  }

 else if (scaletype == 3) {

   // Convert the RA from decimal degrees to seconds. This will catch both positive values RA e.g. 240 deg and negative ones as well e.g. -120 deg as we are limiting to the 24 hours later.

   conv_dec = dec * 3600;
   conv_dec += (2*yshift*cellsize); // This needs to be double to correct for the 1/2 in determining the min and max values later

   conv_ra = 240*(360 + ra);

   conv_ra +=  2 *((wholecirclesecs*cos(degtorad * (dec+(yshift*(cellsize/3600)) ) ) / 360)  * (((0-xshift) * cellsize)/3600) ); // We change the sign of xshift here to account for the fact RA goes right to left

   cpgenv(istart-border, iend+border, jstart-border, jend+border, 1, -1);

    // Determing the axis values for the given cellsize
 
   xstartval = conv_ra + ( ( ( (iend+border) - (istart-border) ) / 2 ) * ( ( (wholecirclesecs*cos(degtorad * dec) ) / 360)  * (cellsize/3600) ) );
   xendval = conv_ra - ( ( ( (iend+border) - (istart-border) ) / 2 ) * ( ( (wholecirclesecs*cos(degtorad * dec) ) / 360)  * (cellsize/3600) ) );

   ystartval = conv_dec - ( ( ( (jend+border) - (jstart-border) ) / 2 ) * cellsize);
   yendval = conv_dec + ( ( ( (jend+border) - (jstart-border) ) / 2 ) * cellsize);



   // printf("RA: %.3e Conv_RA: %.3e DEC: %.3e Conv_DEC: %.3e xstartval: %.3e

   // Set to range in arcseconds
   cpgswin(xstartval, xendval, ystartval, yendval);

    // Draw the axis box
    if ( (axismarks == 1)){
      cpgtbox("XBCZHNTSYO", 0.0, 0.0, "BCZDNTSYO", 0.0, 0.0);
    }
    else {
      cpgtbox("ZHNTSYO", 0.0, 0.0, "ZDNTSYO", 0.0, 0.0);
    }

  }

  else {

    if ( (axismarks == 1)){
      cpgenv(istart-border, iend+border, jstart-border, jend+border, 1, 0);
    }

    else {
      cpgenv(istart-border, iend+border, jstart-border, jend+border, 1, -1);
    }
  }


  if (labels == 1) {

    if (scaletype == 1) {
      sprintf(xlabel,"Angular Size (Arcseconds)");
      sprintf(ylabel,"Angular Size (Arcseconds)");
    }

    else if (scaletype == 2) {
      sprintf(xlabel,"Angular Size");
      sprintf(ylabel,"Angular Size");
    }

    else if (scaletype == 3) {
      sprintf(xlabel,"Right Ascension");
      sprintf(ylabel,"Declination");
    }

    else {
      sprintf(xlabel,"Pixels");
      sprintf(ylabel,"Pixels");
    }

  }
  else {
    sprintf(xlabel," ");
    sprintf(ylabel," ");
  }


  cpglab(xlabel, ylabel, maptitle);
 
 // If the enviroment is set to use seconds, change it back to pixels before returning
  if ( (scaletype == 2) || (scaletype == 3) ) {
  cpgswin(istart-border, iend+border, jstart-border, jend+border);
  }

  return 0;

}
